Framework brl.blitz
Import brl.max2d
Import brl.glmax2d
Import brl.random
Import brl.math
Import brl.timer
Import brl.pngloader
Import brl.ramstream

SeedRnd MilliSecs()
Const GW=800,GH=600
Const MAXHEALTH=100,MAXFUEL=250,MAXPOWER=40

Incbin "tank.png"
Incbin "explosion.png"
Incbin "skyrocket.png"

Type Player
	Global img:TImage,imgw,imgh
	Field name$,x,y,health,fuel,angle,missile:Rocket,lastmove=1,won=0
	Function Load()
		img = LoadImage("incbin::tank.png")
		imgw = ImageWidth(img)
		imgh = ImageHeight(img)
		SetImageHandle img, imgw/2, imgh
	EndFunction
	Method reset()
		a=(x<GW/2)*10-5
		While terrain[x]>GH;x:+a;Wend
		health=MAXHEALTH;fuel=MAXFUEL
		y=terrain[x];missile=Null;angle=-90
	EndMethod
	Method isAlive()
		Return health > 0
	EndMethod
	Method Hit(missile:Rocket)
		If missile.x > x-imgw/2 And missile.x < x+imgw/2 And missile.y > y-imgh And missile.y < y Then
			missile.explode
			health:-20
		EndIf
	EndMethod
	Method Draw()
		If health <= 0 Then Return
		SetColor 0,0,0
		DrawText name, x - TextWidth(name)/2, y-imgh-20
		If lastmove<0 Then SetScale -1, 1
		w;DrawImage img, x, y
		SetScale 1,1
		If missile Then missile.Draw
	EndMethod
	Method checkFallthrough()
		For i = -imgw/2 To imgw/2
			If terrain[i+x] < GH Then Return
		Next
		If health > 0 Then
			e:Explosion=New Explosion
			e.Set x, GH-10
			y = GH+imgh
			health=0
		EndIf
	EndMethod
	Method setY(damage=False)
		diff = Abs(y-terrain[x])
		y = terrain[x]
		If y > GH Then checkFallthrough
		If damage And diff > 10 Then
			health :- (diff-10)
			e:Explosion=New Explosion
			e.Set(x,y)
		EndIf
	EndMethod
	Method Move(v)
		If fuel = 0 Then Return
		If v <> 0 Then lastmove=v
		x:+v;a=imgw/2
		If x < a Then x=a ElseIf x > GW-a Then x=GW-a Else fuel:-Abs v
		setY
		Turn 0
	EndMethod
	Method Turn(v)
		If missile And missile.launched Then Return
		If missile = Null Then missile = New Rocket
		angle:+v
		missile.x = x+45*Cos(angle)
		missile.y = y+45*Sin(angle)
		missile.angle = angle
	EndMethod
	Method Hold()
		If missile = Null Or missile.launched Then Return
		DrawBar "",x-40,y-imgh-50,80,8,missile.power,MAXPOWER,$FF0000,$000000
		missile.power:+1
		If missile.power > MAXPOWER Then Launch
	EndMethod
	Method Holding()
		Return missile <> Null And missile.power > 0
	EndMethod
	Method Launch()
		If missile And missile.launched=0 Then
			missile.launched=1
			missile.power:/5.0
			missile.xv=missile.power*Cos(missile.angle)
			missile.yv=missile.power*Sin(missile.angle)
		EndIf
	EndMethod
	Method NoMissile()
		missile.expl=Null;missile=Null
	EndMethod
	Method EndOfTurn()
		Return health <= 0 Or (missile And missile.failed())
	EndMethod
EndType
Type Explosion
	Const EXPLTIME=2000
	Global list:TList=New TList
	Global img:TImage
	Field starttime,endtime,x,y
	Function Load()
		img = LoadImage("incbin::explosion.png")
		MidHandleImage img
	EndFunction
	Method New()
		starttime=MilliSecs()
		endtime=starttime+EXPLTIME
		list.addLast Self
	EndMethod
	Method Set(_x,_y)
		x=_x;y=_y
	EndMethod
	Function DrawAll()
		For e:Explosion = EachIn list;e.Draw;Next
	EndFunction	
	Method Draw()
		If MilliSecs() > endtime Then list.remove Self;Return
		SetAlpha (endtime-MilliSecs())*1.0/EXPLTIME
		DrawImage img,x,y
		SetAlpha 1
	EndMethod
EndType
Type Rocket
	Global img:TImage
	Field x!,y!,angle,power!,launched=0,xv!,yv!,expl:Explosion
	Function Load()
		img = LoadImage("incbin::skyrocket.png")
		MidHandleImage img
	EndFunction
	Method set(_x!,_y!,_angle)
		x=_x;y=_y;angle=_angle
	EndMethod
	Method Draw()
		SetRotation angle+90
		If launched Then
			x :+ xv
			y :+ yv
			SetRotation 1
			If x >= 0 And x < GW And y > terrain[x] Then explode
			pl[0].Hit Self
			pl[1].Hit Self
			SetRotation ATan2(yv,xv)+90
			yv :+ 0.07
		EndIf
		DrawImage img,x,y
		SetRotation 0
	EndMethod
	Method explode()
		If expl Then Return
		expl = New Explosion
		expl.Set x,y
		Krater x
		yv = -1
		y = GH+42
	EndMethod
	Method Failed()
		Return x < -40 Or x > GW+40 Or y > GH+40
	EndMethod
EndType

Function DrawBar(name$,x,y,w,h,val,maxval,fore,back,tc=0,flipped=False)
	SetAlpha 0.5
	length=1.0*w*val/maxval
	SetColor (back Shr 16) & $FF, (back Shr 8) & $FF, back & $FF
	If flipped Then
		DrawRect x,y,w-length,h
		If val > 0 Then
			SetColor (fore Shr 16) & $FF, (fore Shr 8) & $FF, back & $FF
			DrawRect x+w-length,y,length,h
		EndIf
	Else
		DrawRect x+length,y,w-length,h
		If val > 0 Then
			SetColor (fore Shr 16) & $FF, (fore Shr 8) & $FF, back & $FF
			DrawRect x,y,length,h
		EndIf
	EndIf
	SetAlpha 1
	If name Then
		SetColor (tc Shr 16) & $FF, (tc Shr 8) & $FF, tc & $FF
		If flipped Then
			DrawText name, x+w-TextWidth(name)-2, y+(h-TextHeight(name))/2
		Else
			DrawText name, x+2, y+(h-TextHeight(name))/2
		EndIf	
	EndIf
EndFunction

Function NewTerrain()
	Const GW360#=360.0/GW
	Local a#[3],b#[3]
	tcolor=Rnd()*3
	m=(tcolor+1)*50
	y = GH-m-100
	For i = 0 To 2
		a[i] = Rand(m)-m/2
		b[i] = Rand(m)-m/2
	Next
	For i = 0 To 799
		c# = y
		For j = 0 To 2
			c :+ a[j]*Cos((j+1)*i*GW360)+b[j]*Sin((j+1)*i*GW360)
		Next
		terrain[i] = c
	Next
EndFunction
Function DrawTerrain()
	For i = 0 To 799
		j = 2*tcolor + (i Mod 2)
		SetColor cmap[j][0],cmap[j][1],cmap[j][2]
		DrawRect i, terrain[i], 1, GW-terrain[i]
	Next
EndFunction

Function Krater(x1)
	d=Rand(20,100)
	For i = -45 To 45
		x = x1+i
		If x >= 0 And x < GW Then
			terrain[x] :- d*Sin(2*i-90)
		EndIf
	Next
	pl[1].setY True
	pl[0].setY True
EndFunction

Const	L0$="[F1] for help",..
		L1$="Press and Hold [Space] to launch missile",..
		L2$="The longer you hold [Space] the faster the rocket will go",..
		L3$="Move tank with [A]/[Left Arrow] and [D]/[Right Arrow]",..
		L4$="Turn missile with [W]/[Up Arrow] and [S]/[Down Arrow]",..
		L5$="Press [Space] to play again!"
		
Global cmap[][] = [[255,255,120],[180,180,120],[100,255,100],[100,180,100],[255,255,255],[220,220,230]]
Global terrain[GW],tcolor,gamemode=0,onturn=0
timer = CreateTimer(50)
Global pl:Player[] = [New Player, New Player]
AppTitle="Rocket Tanks v1.1"
Graphics GW,GH
SetBlend ALPHABLEND
SetClsColor 120,180,255

Explosion.Load
Player.Load
pl[0].name="Player 1"
pl[1].name="Player 2"
Rocket.Load

Repeat

	If gamemode = 0 Then
		NewTerrain
		onturn = 0
		pl[0].x = Rand(40,100)
		pl[1].x = Rand(GW-100, GW-40)
		pl[0].reset
		pl[1].reset
		pl[onturn].Turn 0
		gamemode = 1
		FlushKeys
		Delay 100
	EndIf

	Cls
	DrawTerrain
	pl[0].Draw
	pl[1].Draw

	If KeyDown(KEY_SPACE) Then pl[onturn].Hold

	If Not pl[onturn].Holding() Then
		If KeyDown(KEY_LEFT) Or KeyDown(KEY_A) Then
			pl[onturn].Move -1
		ElseIf KeyDown(KEY_RIGHT) Or KeyDown(KEY_D) Then
			pl[onturn].Move 1
		ElseIf KeyDown(KEY_UP) Or KeyDown(KEY_W) Then
			pl[onturn].Turn -1
		ElseIf KeyDown(KEY_DOWN) Or KeyDown(KEY_S) Then
			pl[onturn].Turn 1
		EndIf
	ElseIf Not KeyDown(KEY_SPACE) Then
		pl[onturn].Launch
	EndIf
	
	If KeyHit(KEY_F10) Then gamemode=0
	
	Explosion.DrawAll
	
	s$ = pl[onturn].name+" on turn!"
	w;DrawText s,(GW-TextWidth(s))/2,50
	DrawText L0,(GW-TextWidth(L0))/2,30

	DrawText pl[0].name,2,2
	DrawText pl[1].name,GW-TextWidth(pl[1].name)-2,2
	DrawBar "Health",0,20,200,18,pl[0].health,MAXHEALTH,$FF0000,$773300,$FFFFFF
	DrawBar "Health",GW-200,20,200,18,pl[1].health,MAXHEALTH,$FF0000,$773300,$FFFFFF,True
	DrawBar "Fuel",0,38,200,18,pl[0].fuel,MAXFUEL,$AAAAAA,$555577,$FFFFFF
	DrawBar "Fuel",GW-200,38,200,18,pl[1].fuel,MAXFUEL,$AAAAAA,$555577,$FFFFFF,True

	help
	If pl[onturn].EndOfTurn() And Explosion.list.isEmpty() And Not KeyDown(KEY_SPACE) Then
		pl[onturn].NoMissile
		onturn=1-onturn
		pl[onturn].Turn 0
	
		If Not(pl[0].isAlive() And pl[1].isAlive()) Then
			SetColor 0,0,0
			SetAlpha 0.75
			DrawRect 0,0,GW,GH
			SetAlpha 1
			
			If pl[0].isAlive() Then
				s$ = pl[0].name+" has won!"
				pl[0].won:+1
			Else
				s$ = pl[1].name+" has won!"
				pl[1].won:+1
			EndIf
			stats
			w;DrawText s,(GW-TextWidth(s))/2,50
			DrawText L5,(GW-TextWidth(L5))/2,70
			Flip
			FlushKeys
			Delay 200
			Repeat
				If AppTerminate() Or KeyHit(KEY_ESCAPE) Then End
				Delay 100
			Until KeyHit(KEY_SPACE)
			gamemode=0
		EndIf
	EndIf

	Flip 0
	WaitTimer timer

Until AppTerminate()
End

Function stats()
	s$="Stats"
	w;DrawText s,(GW-TextWidth(s))/2,GH/2+20
	s$=pl[0].name+" : "+pl[0].won
	DrawText s,(GW-TextWidth(s))/2,GH/2+40
	s$=pl[1].name+" : "+pl[1].won
	DrawText s,(GW-TextWidth(s))/2,GH/2+58
	DrawText "Images from pixabay.com",2,GH-20
EndFunction

Function help()
	Global enabled=False
	If KeyHit(KEY_F1) Then enabled=Not enabled
	If enabled Then
		If KeyHit(KEY_ESCAPE) Then enabled=False
		SetColor 0,0,0
		SetAlpha 0.75
		DrawRect 0,0,GW,GH
		SetAlpha 1
		w;DrawText L1,(GW-TextWidth(L1))/2,GH/2-80
		DrawText L2,(GW-TextWidth(L2))/2,GH/2-60
		DrawText L3,(GW-TextWidth(L3))/2,GH/2-40	
		DrawText L4,(GW-TextWidth(L4))/2,GH/2-20	

		stats
	EndIf	
EndFunction

Function w()
	SetColor 255,255,255
EndFunction